<?php

namespace Controller;

use Traits\TResponse;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use Util\InstanceProvider;
use Service\ProductVersion as ProductVersionService;

class ProductVersion
{
    use TResponse;

    /**
     * @var ProductVersionService
     */
    private $productVersionService;

    public function __construct(ProductVersionService $productVersionService)
    {
        $this->productVersionService = $productVersionService;
    }

    public function findProductVersion(Request\Filter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        try {
            $versions = $this->productVersionService->findProductVersion($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('findProductVersion', $versions));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findProductVersionLtsWithoutBuild(Request $request, Response $response)
    {
        try {
            $productId =  $request->getParameter('PRODUCT_ID');
            $resourceId =  $request->getParameter('RESOURCE_ID');
            $versions = $this->productVersionService->findProductVersionLtsWithoutBuild($productId, $resourceId);
            $this->sendResponse($versions, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findLtsCurrentProductVersionFromVersion(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $version = InstanceProvider::getService()->getProductVersion()->findLtsCurrentProductVersionFromVersion($conditions);
            $response->addDataSet(new DataSet('findLtsCurrentProductVersionFromVersion', $version));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findProductVersionLtsByProject(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $version = InstanceProvider::getService()->getProductVersion()->findProductVersionLtsByProject($conditions);
            $response->addDataSet(new DataSet('findProductVersionLtsByProject', $version));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findNextLTSFromVersion(Request\Filter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $page       = $request->getFilterCriteria()->getPage();
        $pageSize   = $request->getFilterCriteria()->getPageSize();
        try {
            $version = InstanceProvider::getService()->getProductVersion()->findNextLTSFromVersion($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('findNextLTSFromVersion', $version));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}
