<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities AS DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Devops\Entities as EntitiesDevops;
use Util\Utilities;

class Structure
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    const IS_MANAGER = Entities\Permission::TYPE_MANAGER;
    const IS_RESOURCE = Entities\Permission::TYPE_RESOURCE;

    public function __construct(EntityManager $entityManager)
    {
         $this->entityManager = $entityManager;
    }

    public function findStructureFieldNext()
    {
        $return = InstanceProvider::getRepository()->getLocalStructure()->findStructureFieldNext();
        $return['data'] = Utilities::fixParentsForTreeGrid($return['data']);
        return $return;
    }

    public function getAllStructuresChildrens($structureIds){
        return InstanceProvider::getRepository()->getStructure()->getAllStructuresChildrens($structureIds);
    }

    public function getAllStructuresChildrensByManager($resourceIds){
        return InstanceProvider::getRepository()->getStructure()->getAllStructuresChildrensByManager($resourceIds);
    }

    
    /**
     * @param $structureId
     * @param $resources
     */
    public function setManager($structureId, $resources)
    {
        $permissionRepository = InstanceProvider::getRepository()->getPermission();
        $this->removeNotFoundPermission($structureId, $resources, Structure::IS_MANAGER);
        foreach ($resources as $eachResource) {
            $permission = $permissionRepository->findOneBy(array('resource' => $eachResource, 'structure' => $structureId,
                'type' => Structure::IS_MANAGER));
            $this->persistPermission($permission, $eachResource, $structureId, Structure::IS_MANAGER);
        }
    }

    function removeNotFoundPermission($structure, $resources, $type){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission[] $permissions */
        $permissionRepository = InstanceProvider::getRepository()->getPermission();

        $permissions = $permissionRepository->findBy(array('structure' => $structure,'type'=>$type));
        foreach($permissions as $permission){
            if(!in_array($permission->getResource(), $resources)){
                $this->entityManager->remove($permission);
            }
        }
        $this->entityManager->flush();
    }

    function persistPermission($permission = null, $resource, $structure, $type = 'R'){
        $resourceModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource';
        $structureModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure';
        if(!($resource instanceof $resourceModel)){
            $resource = $this->entityManager->find($resourceModel, $resource);
        }
        if(!($structure instanceof $structureModel)){
            $structure = $this->entityManager->find($structureModel, $structure);
        }
        if(empty($permission)){
            return $this->createPermission($resource, $structure, $type);
        }
        else{
            return $this->updatePermission($permission, $resource, $structure, $type);
        }
    }

    function createPermission($resource, $structure, $type = self::IS_RESOURCE){
        $permissionFactory = InstanceProvider::getFactory()->getPermission();
        $permission = $permissionFactory->createPermission($resource, $structure, $type);
        $this->entityManager->persist($permission);
        $this->entityManager->flush();
        return $permission;
    }

    function updatePermission($permission, $resource, $structure, $type = self::IS_RESOURCE){
        $permissionFactory = InstanceProvider::getFactory()->getPermission();
        $permissionModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission';
        if(!($permission instanceof $permissionModel)){
            $permission = $this->entityManager->find($permissionModel, $permission);
        }
        $permission = $permissionFactory->updatePermission($permission, $resource, $structure, $type);
        $this->entityManager->persist($permission);
        $this->entityManager->flush();
        return $permission;
    }
}
