<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class TaskTracking
{

    private $entityManager;
    private $environment;

    /**
     * TaskTracking constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {   
        $this->environment = InstanceProvider::getEnvironment();
        $this->entityManager = $entityManager;
    }

    public function findTaskTracking($dynamicFilter,$search, $searchIn, $find, $order, $page, $limit){
        $filters = json_decode(utf8_encode(base64_decode($dynamicFilter)),true);
        $project = false;
        $statusResource = false;
        $addFilter = array();
        $filterLabel = '';
        $filterIssue = '';
        $conditionsSearchQuery = '';
        foreach ($filters as $key => &$filter){
            if($key == 'LATE') {
                $late = $filter[0]['value'];
                unset($filters[$key]);
            }
            if($key == 'TASK_NAME_FILTER') {
                $filter[0]['value'] = str_replace("'", "''", $filter[0]['value']);
            }
            if($key == 'PROJECT_ID'){
                $project = false;
            }
            else if($key == 'STRUCTURE'){
                $filter[0]['value'] = DevOpsUtilities::getArrayFromArrayAssociative(InstanceProvider::getService()->getStructure()->getAllStructuresChildrens($filter[0]['value']));
            }else if($key == 'PERIOD'){
                $addFilter = array(
                    'PERIOD_END' =>
                    $filter);
                unset($filters[$key]);
            }else if($key == 'RESPONSIBLE'){
                $structures = DevOpsUtilities::getArrayFromArrayAssociative(InstanceProvider::getService()->getStructure()->getAllStructuresChildrensByManager($filter[0]['value']));
                $filters['STRUCTURE'] = array(array(
                    'value'=>$structures,
                    'operation'=>'IN',
                ));
                unset($filters[$key]);
            }else if($key == 'LABEL') {
                $values = $filter[0]['value'];
                $filterLabel = implode(",", $values);
                unset($filters[$key]);
            }else if($key == 'ISSUE') {
                $values = $filter[0]['value'];
                $filterIssue = implode(",", $values);
                unset($filters[$key]);
            }else if($key == 'STATUS_RESOURCE') {
                $statusResource = $filter[0]['value'];
                unset($filters[$key]);
            }
        }
        $filters = array_merge($filters,$addFilter);
        $filterString = InstanceProvider::getRepository()->getLocalTask()->createDynamicFilter($filters, 'WHERE 1=1', false);
        if(!empty($late)){
            $lateTasks = InstanceProvider::getRepository()->getTask()->getTotalDelayedTasksByCurrentUser($this->environment->getCurrentUserId(),true);
            if(!empty($lateTasks)) {
                $tasks = implode(",", Utilities::getArrayFromAssociativeArray($lateTasks));
                $filterString .= $late == 'Y' ? " AND TASK_ID IN ($tasks)" : "AND TASK_ID NOT IN ($tasks)";
            }
        }
        if(!empty($search) && !empty($searchIn)){
            $conditionsSearchQuery = InstanceProvider::getRepository()->getLocalTask()->createSearch($search, $searchIn);
        }
        $tasks = InstanceProvider::getRepository()->getLocalTask()->getAllTasksByProjectOnDemand($this->environment->getCurrentOrganizationId(),
        $filterString, $filterLabel, $filterIssue, $project,$statusResource, $conditionsSearchQuery, $page, $limit);
        foreach($tasks["data"] as &$task){
            if(empty($task['_hasChild'])){
                if(empty($task['TOTAL_APPROPRIATION']  )) {
                    $task['TOTAL_APPROPRIATION'] = 0;
                }
            }
        }
        return $tasks;
    }

    public function getInfoTaskInTaskTracking($idTask, $resource = null) {
        $taskRepository = InstanceProvider::getRepository()->getLocalTask();
		$infoTask = $taskRepository->getInfoTaskInTaskTracking($idTask, $this->environment->getCurrentOrganizationId(), $resource);
        $taskRelationshipRepository = InstanceProvider::getRepository()->getTaskRelationship();
        foreach($infoTask as &$task) {
            $task['IS_RELATED_TASK'] = false;
            $task['USER_ID'] = strval($this->environment->getCurrentUserId());
            $relatedTasks = $taskRelationshipRepository->findRelatedTaskIds($task['ID']);
            if(!empty($relatedTasks)) {
                $mainProject = InstanceProvider::getRepository()->getTask()->find($relatedTasks['MAIN_TASK'])->getProject()->getId();
                $task['MAIN_TASK'] = $relatedTasks['MAIN_TASK'];
                $task['MAIN_PROJECT'] = $mainProject;
                $task['IS_RELATED_TASK'] = true;
            }
        }
        return $infoTask;
	}

    public function findAllStatusTaskTracking() {
		return InstanceProvider::getRepository()->getLocalTask()->findAllStatusTaskTracking($this->environment->getCurrentOrganizationId());
	}

    public function findAllIssuesFromFilterTaskTracking() {
		return InstanceProvider::getRepository()->getLocalTask()->getAllIssuesFromFilterTaskTracking($this->environment->getCurrentOrganizationId());
	}

    public function findLabelsByProjectTaskTracking(){
        
        return InstanceProvider::getRepository()->getLocalTask()->findLabelsByProjectTaskTracking($this->environment->getCurrentOrganizationId(),$this->environment->getCurrentUserId());
    }

    public function associateTaskAndLabels($taskId, $labelIds, $type = null, $meetingProjectIds = null){
        $task = InstanceProvider::getRepository()->getTask()->find($taskId);
        $labelRepository = InstanceProvider::getRepository()->getLabel();
        $labelsIsCurrentUser = $labelRepository->findBy(array('resource' => $this->environment->getCurrentUserId()));
        if($meetingProjectIds) {
            $meetingProjectIds = $this->getImInMeetingProjectIds();
            $labelsIsCurrentUser = array_merge($labelsIsCurrentUser, $labelRepository->findBy(array('project' => $meetingProjectIds)));
        }
        $taskLabelRepository = InstanceProvider::getRepository()->getTaskLabel();
        foreach ($labelIds as $labelId) {
            if(!empty($labelId)){
                /** @var  Entities\Label $label */
                $label = $labelRepository->find($labelId);
                $taskLabel = $taskLabelRepository->findBy(array("task" =>$taskId, "label" => $label->getId(),"type" => $type));

                $taskLabelFactory = InstanceProvider::getFactory()->getTaskLabel();
                if (empty($taskLabel)) {
                    $taskLabel = $taskLabelFactory->createTaskLabel($task, $label, $type);
                    $this->entityManager->persist($taskLabel);
                }
            }
        }

        $this->entityManager->flush();
        /** @var Entities\TaskLabel[] $taskLabels */
        $taskLabels = $taskLabelRepository->findBy(array('label'=>$labelsIsCurrentUser,"task" => $task->getId(),"type" => $type));
        foreach ($taskLabels as $taskLabel) {
            if(!in_array($taskLabel->getLabel()->getId(), $labelIds)){
                $this->entityManager->remove($taskLabel);
                $this->entityManager->flush();
            }
        }
    }

    public function saveTaskReponse($taskId, $taskResponse){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $task = InstanceProvider::getRepository()->getTask()->find($taskId);
            $task->setResponse($taskResponse);
            $this->entityManager->persist($task);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

     /**
     * @return array
     */
    public function getImInMeetingProjectIds()
    {
        return InstanceProvider::getService()->getIssue()->getImInMeetingProjectIds();
    }

}